# -*- coding: utf-8 -*-

# Vendor Library
from spyne.decorator import rpc
from spyne.model.primitive import Unicode
from spyne.model.complex import Array

# ProjectLibrary
from kinder.webservice.api import declaration as decl_api
from kinder.webservice.api.exceptions import ApiException
from kinder.webservice.spyne_ws.exceptions import SpyneException

from kinder.webservice.spyne_ws.declaration_info.types import (
    DeclarantPersonal, ChildPersonal)
from kinder.webservice.spyne_ws.declaration_info import helpers
from kinder.webservice.spyne_ws.declaration_info.errors import (
    DeclCheckErrors,
    DeclCheckException
)
from concentrator.webservice.helpers import get_binary_data
from concentrator.webservice.entities import (
    NewApplicationResponse, GetApplicationRequest)
from concentrator.webservice.proxy import GetApplicationProxy
from lipetsk_specifics.webservice.entities import (
    NewApplicationRequest, GetApplicationResponse,
    UpdateApplicationRequest, UpdateApplicationResponse,
    DeclarationStatus)
from lipetsk_specifics.webservice.proxy import (
    LipetskNewApplicationProxy, LipetskUpdateApplicationProxy)
from lipetsk_specifics.webservice.spyne_objects_proxy import (
    LipetskDeclaredPersonDataProxy, LipetskApplicantDataProxy,
    LipetskEduOrganizationsDataProxy)
from lipetsk_specifics.webservice.helpers import RegionalCheckDeclarationProcess



# Методы концентратора
# - Передача Заявления в региональную систему предоставления услуг
#   NewApplication
@rpc(
    NewApplicationRequest,
    _returns=NewApplicationResponse,
    _body_style='bare',
    _out_message_name='NewApplicationResponse')
def NewApplicationRequest(
        self, NewApplicationRequest):
    binary_data, request_code = get_binary_data(self.udc)
    result = LipetskNewApplicationProxy(
        NewApplicationRequest, binary_data, request_code).process()
    return NewApplicationResponse(RegionalId=str(result))


# Получение данных Заявления для изменения
# GetApplication
@rpc(GetApplicationRequest,
     _returns=GetApplicationResponse,
     _body_style='bare',
     _out_message_name='GetApplicationResponse')
def GetApplicationRequest(ctx, request):

    ExternalId = request.ExternalId
    try:
        decl = decl_api.get_decl_by_external_id(ExternalId)
    except ApiException as exc:
        raise SpyneException(exc.code, exc.message)

    decl_proxy = LipetskDeclaredPersonDataProxy(decl)
    app_proxy = LipetskApplicantDataProxy(decl)
    dou_proxy = LipetskEduOrganizationsDataProxy(decl)
    return GetApplicationProxy(
        decl, decl_proxy, app_proxy, dou_proxy).process()


# - Изменение Заявления в региональной системе предоставления услуг
@rpc(
    UpdateApplicationRequest,
    _returns=UpdateApplicationResponse,
    _body_style='bare',
    _out_message_name='UpdateApplicationResponse')
def UpdateApplicationRequest(
        ctx, request):
    ExternalId = request.ExternalId
    try:
        decl = decl_api.get_decl_by_external_id(ExternalId)
    except ApiException as exc:
        raise SpyneException(exc.code, exc.message)

    binary_data, request_code = get_binary_data(ctx.udc)
    return LipetskUpdateApplicationProxy.process(
        request, decl, binary_data=binary_data, request_code=request_code)


@rpc(Unicode(nillable=False, min_occurs=1),
     DeclarantPersonal,
     ChildPersonal.customize(nillable=False, min_occurs=1),
     _in_variable_names={
         "okato": "OKATO",
         "declarant_personal": "DeclarantPersonal",
         "child_personal": "ChildPersonal"},
     _in_message_name="CheckDeclarationDOURequest",
     _returns=Array(DeclarationStatus))
def CheckDeclarationDOU(ctx, okato, declarant_personal, child_personal):

    results = []

    try:
        declarations = helpers.find_declarations(
            child_personal, True, okato
        )
    except DeclCheckException as e:
        err_code = e.message
        raise SpyneException(
            code=err_code,
            message=DeclCheckErrors.ERR_CODES[err_code][1]
        )

    for declaration in declarations:

        results.extend(
            RegionalCheckDeclarationProcess(
                declaration).process())

    return results
